@@ -44,7 +44,11 @@ enum
   GCONF_PROP_0,
   GCONF_PROP_COUNTDOWN,
   GCONF_PROP_WEBCAM,
-  GCONF_PROP_SELECTED_EFFECTS
+  GCONF_PROP_SELECTED_EFFECTS,
+  GCONF_PROP_X_RESOLUTION,
+  GCONF_PROP_Y_RESOLUTION,
+  GCONF_PROP_VIDEO_PATH,
+  GCONF_PROP_PHOTO_PATH
 };
 
 GType        cheese_gconf_get_type(void);
